#!/usr/bin/env Rscript

# OHCRN-LEI - LLM-based Extraction of Information
# Copyright (C) 2025 Ontario Institute for Cancer Research

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

# merge.R reads tsv tables of TP/FP/FN counts generated by compare.py
# and sums over them by category. It is called by evaluation.sh
# First argument is the path to the input directory (containing the TSVs)
# Second argument is the path to the desired output file.

args <- commandArgs(TRUE)
indir <- args[[1]]
outfile <- args[[2]]

#read input files into list of data.frames
infiles <- list.files(path = indir, pattern = ".tsv$", full.names = TRUE)
data_list <- lapply(infiles, read.delim)
#derive document names
docnames <- sub("\\.tsv$", "", basename(infiles))

#feed data.frames into 3D array
mat <- array(0,
  dim = c(nrow(data_list[[1]]), 3, length(data_list)),
  dimnames = list(rownames(data_list[[1]]), c("tp", "fn", "fp"), docnames)
)
for (i in 1:length(data_list)) {
  mat[, , i] <- as.matrix(data_list[[i]][, 1:3])
}
#sum over 3rd dimension
sums <- apply(mat, 1:2, sum)
#write result to output file
write.table(sums, outfile, sep = "\t", quote = FALSE)
